@@ -309,7 +309,7 @@ g_daemon_file_hash (GFile *file)
 
   return
     g_str_hash (daemon_file->path) ^
-    (guint32)daemon_file->mount_spec;  /* We have unique mount_spec objects so hash directly on it */
+    GPOINTER_TO_UINT (daemon_file->mount_spec);  /* We have unique mount_spec objects so hash directly on it */
 }
 
 static gboolean
@@ -332,6 +332,14 @@ match_prefix (const char *path, const char *prefix)
   prefix_len = strlen (prefix);
   if (strncmp (path, prefix, prefix_len) != 0)
     return NULL;
+
+  /* Handle the case where prefix is the root, so that
+   * the IS_DIR_SEPRARATOR check below works */
+  if (prefix_len > 0 &&
+      prefix[prefix_len-1] == '/')
+    prefix_len--;
+  
+  
   return path + prefix_len;
 }
 
@@ -821,19 +829,22 @@ query_info_async_cb (DBusMessage *reply,
 static void
 g_daemon_file_query_info_async (GFile                      *file,
 				const char                 *attributes,
-				GFileQueryInfoFlags           flags,
+				GFileQueryInfoFlags         flags,
 				int                         io_priority,
 				GCancellable               *cancellable,
 				GAsyncReadyCallback         callback,
 				gpointer                    user_data)
 {
+  guint32 dbus_flags;
+
+  dbus_flags = flags;
   do_async_path_call (file,
 		      G_VFS_DBUS_MOUNT_OP_QUERY_INFO,
 		      cancellable,
 		      callback, user_data,
 		      query_info_async_cb, NULL, NULL,
 		      DBUS_TYPE_STRING, &attributes,
-		      DBUS_TYPE_UINT32, &flags,
+		      DBUS_TYPE_UINT32, &dbus_flags,
 		      0);
 }
 
@@ -1198,15 +1209,14 @@ mount_mountable_async_cb (DBusMessage *reply,
   char *path;
   DBusMessageIter iter;
   GFile *file;
-  dbus_bool_t must_mount_location;
+  dbus_bool_t must_mount_location, is_uri;
 
   path = NULL;
-  
+
   dbus_message_iter_init (reply, &iter);
-  mount_spec = g_mount_spec_from_dbus (&iter);
   
-  if (mount_spec == NULL ||
-      !_g_dbus_message_iter_get_args (&iter, NULL,
+  if (!_g_dbus_message_iter_get_args (&iter, NULL,
+				      DBUS_TYPE_BOOLEAN, &is_uri,
 				      G_DBUS_TYPE_CSTRING, &path,
 				      DBUS_TYPE_BOOLEAN, &must_mount_location,
 				      0))
@@ -1215,26 +1225,44 @@ mount_mountable_async_cb (DBusMessage *reply,
 				       G_IO_ERROR, G_IO_ERROR_FAILED,
 				       _("Invalid return value from call"));
       g_simple_async_result_complete (result);
+
+      return;
+    }
+
+  if (is_uri)
+    {
+      file = g_file_new_for_uri (path);
     }
   else
     {
+      mount_spec = g_mount_spec_from_dbus (&iter);
+      if (mount_spec == NULL)
+	{
+	  g_simple_async_result_set_error (result,
+					   G_IO_ERROR, G_IO_ERROR_FAILED,
+					   _("Invalid return value from call"));
+	  g_simple_async_result_complete (result);
+	  return;
+	}
+      
       file = g_daemon_file_new (mount_spec, path);
       g_mount_spec_unref (mount_spec);
-      g_free (path);
-      g_simple_async_result_set_op_res_gpointer (result, file, g_object_unref);
-
-      if (must_mount_location)
-	{
-	  g_file_mount_enclosing_volume (file,
-					 mount_operation,
-					 cancellable,
-					 mount_mountable_location_mounted_cb,
-					 g_object_ref (result));
-	  
-	}
-      else
-	g_simple_async_result_complete (result);
     }
+  
+  g_free (path);
+  g_simple_async_result_set_op_res_gpointer (result, file, g_object_unref);
+
+  if (must_mount_location)
+    {
+      g_file_mount_enclosing_volume (file,
+				     mount_operation,
+				     cancellable,
+				     mount_mountable_location_mounted_cb,
+				     g_object_ref (result));
+      
+    }
+  else
+    g_simple_async_result_complete (result);
 }
 
 static void
@@ -1283,6 +1311,83 @@ g_daemon_file_mount_mountable_finish (GFile               *file,
   return NULL;
 }
 
+static void
+eject_mountable_async_cb (DBusMessage *reply,
+			  DBusConnection *connection,
+			  GSimpleAsyncResult *result,
+			  GCancellable *cancellable,
+			  gpointer callback_data)
+{
+  g_simple_async_result_complete (result);
+}
+
+static void
+g_daemon_file_eject_mountable (GFile               *file,
+			       GMountUnmountFlags   flags,
+			       GCancellable        *cancellable,
+			       GAsyncReadyCallback  callback,
+			       gpointer             user_data)
+{
+  guint32 dbus_flags;
+
+  dbus_flags = flags;
+  do_async_path_call (file,
+		      G_VFS_DBUS_MOUNT_OP_EJECT_MOUNTABLE,
+		      cancellable,
+		      callback, user_data,
+		      eject_mountable_async_cb,
+		      NULL, NULL,
+		      DBUS_TYPE_UINT32, &dbus_flags,
+		      0);
+}
+
+static gboolean
+g_daemon_file_eject_mountable_finish (GFile               *file,
+				      GAsyncResult        *result,
+				      GError             **error)
+{
+  return TRUE;
+}
+
+static void
+unmount_mountable_async_cb (DBusMessage *reply,
+			    DBusConnection *connection,
+			    GSimpleAsyncResult *result,
+			    GCancellable *cancellable,
+			    gpointer callback_data)
+{
+  g_simple_async_result_complete (result);
+}
+
+static void
+g_daemon_file_unmount_mountable (GFile               *file,
+				 GMountUnmountFlags   flags,
+				 GCancellable        *cancellable,
+				 GAsyncReadyCallback  callback,
+				 gpointer             user_data)
+{
+  guint32 dbus_flags;
+
+  dbus_flags = flags;
+  do_async_path_call (file,
+		      G_VFS_DBUS_MOUNT_OP_UNMOUNT_MOUNTABLE,
+		      cancellable,
+		      callback, user_data,
+		      eject_mountable_async_cb,
+		      NULL, NULL,
+		      DBUS_TYPE_UINT32, &dbus_flags,
+		      0);
+}
+
+static gboolean
+g_daemon_file_unmount_mountable_finish (GFile               *file,
+					GAsyncResult        *result,
+					GError             **error)
+{
+  return TRUE;
+}
+
+
 typedef struct {
   GFile *file;
   GMountOperation *mount_operation;
@@ -1845,7 +1950,8 @@ g_daemon_file_move (GFile                  *source,
 static GFileMonitor*
 g_daemon_file_monitor_dir (GFile* file,
 			   GFileMonitorFlags flags,
-			   GCancellable *cancellable)
+			   GCancellable *cancellable,
+			   GError **error)
 {
   GFileMonitor *monitor;
   char *obj_path;
@@ -1859,7 +1965,7 @@ g_daemon_file_monitor_dir (GFile* file,
   reply = do_sync_path_call (file, 
 			     G_VFS_DBUS_MOUNT_OP_CREATE_DIR_MONITOR,
 			     &mount_info, NULL,
-			     cancellable, NULL,
+			     cancellable, error,
 			     DBUS_TYPE_UINT32, &flags_dbus,
 			     0);
   
@@ -1876,7 +1982,8 @@ g_daemon_file_monitor_dir (GFile* file,
     {
       g_mount_info_unref (mount_info);
       dbus_message_unref (reply);
-      g_warning ("Invalid return value from monitor_dir");
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+		   _("Invalid return value from monitor_dir"));
       return NULL;
     }
   
@@ -1892,7 +1999,8 @@ g_daemon_file_monitor_dir (GFile* file,
 static GFileMonitor*
 g_daemon_file_monitor_file (GFile* file,
 			    GFileMonitorFlags flags,
-			    GCancellable *cancellable)
+			    GCancellable *cancellable,
+			    GError **error)
 {
   GFileMonitor *monitor;
   char *obj_path;
@@ -1906,7 +2014,7 @@ g_daemon_file_monitor_file (GFile* file,
   reply = do_sync_path_call (file, 
 			     G_VFS_DBUS_MOUNT_OP_CREATE_FILE_MONITOR,
 			     &mount_info, NULL,
-			     cancellable, NULL,
+			     cancellable, error,
 			     DBUS_TYPE_UINT32, &flags_dbus,
 			     0);
   
@@ -1923,7 +2031,8 @@ g_daemon_file_monitor_file (GFile* file,
     {
       g_mount_info_unref (mount_info);
       dbus_message_unref (reply);
-      g_warning ("Invalid return value from monitor_dir");
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+		   _("Invalid return value from monitor_file"));
       return NULL;
     }
   
@@ -1969,6 +2078,10 @@ g_daemon_file_file_iface_init (GFileIface *iface)
   iface->mount_enclosing_volume_finish = g_daemon_file_mount_enclosing_volume_finish;
   iface->mount_mountable = g_daemon_file_mount_mountable;
   iface->mount_mountable_finish = g_daemon_file_mount_mountable_finish;
+  iface->unmount_mountable = g_daemon_file_unmount_mountable;
+  iface->unmount_mountable_finish = g_daemon_file_unmount_mountable_finish;
+  iface->eject_mountable = g_daemon_file_eject_mountable;
+  iface->eject_mountable_finish = g_daemon_file_eject_mountable_finish;
   iface->query_filesystem_info = g_daemon_file_query_filesystem_info;
   iface->set_display_name = g_daemon_file_set_display_name;
   iface->delete_file = g_daemon_file_delete;
