@@ -48,6 +48,7 @@
 #include "gvfsjobclosewrite.h"
 #include "gvfsjobseekwrite.h"
 #include "gvfsjobsetdisplayname.h"
+#include "gvfsjobmountmountable.h"
 #include "gvfsjobqueryinfo.h"
 #include "gvfsjobdelete.h"
 #include "gvfsjobqueryfsinfo.h"
@@ -263,8 +264,8 @@ update_from_files (GVfsBackendComputer *backend,
               filename = g_strconcat ("/", new->filename, NULL);
               g_vfs_monitor_emit_event (backend->root_monitor,
                                         G_FILE_MONITOR_EVENT_CHANGED,
-                                        backend->mount_spec, filename,
-                                        NULL, NULL);
+                                        filename,
+                                        NULL);
               g_free (filename);
             }
           
@@ -276,8 +277,8 @@ update_from_files (GVfsBackendComputer *backend,
           filename = g_strconcat ("/", old->filename, NULL);
           g_vfs_monitor_emit_event (backend->root_monitor,
                                     G_FILE_MONITOR_EVENT_DELETED,
-                                    backend->mount_spec, filename,
-                                    NULL, NULL);
+                                    filename,
+                                    NULL);
           g_free (filename);
           oldl = oldl->next;
         }
@@ -286,8 +287,8 @@ update_from_files (GVfsBackendComputer *backend,
           filename = g_strconcat ("/", new->filename, NULL);
           g_vfs_monitor_emit_event (backend->root_monitor,
                                     G_FILE_MONITOR_EVENT_CREATED,
-                                    backend->mount_spec, filename,
-                                    NULL, NULL);
+                                    filename,
+                                    NULL);
           g_free (filename);
           newl = newl->next;
         }
@@ -327,7 +328,7 @@ recompute_files (GVfsBackendComputer *backend)
             {
               volume = ll->data;
 
-              file = g_slice_new (ComputerFile);
+              file = g_slice_new0 (ComputerFile);
               file->drive = g_object_ref (drive);
               file->volume = volume; /* Takes ref */
               file->mount = g_volume_get_mount (volume);
@@ -517,7 +518,7 @@ try_mount (GVfsBackend *backend,
                            backend,
                            NULL, 0);
 
-  computer_backend->root_monitor = g_vfs_monitor_new (g_vfs_backend_get_daemon (backend));
+  computer_backend->root_monitor = g_vfs_monitor_new (backend);
   
   recompute_files (computer_backend);
 
@@ -613,6 +614,8 @@ file_info_from_file (ComputerFile *file,
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_EJECT, file->can_eject);
 
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FALSE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE, FALSE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH, FALSE);
 }
 
 static gboolean
@@ -680,6 +683,9 @@ try_query_info (GVfsBackend *backend,
       icon = g_themed_icon_new ("gnome-fs-client");
       g_file_info_set_icon (info, icon);
       g_object_unref (icon);
+      g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FALSE);
+      g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE, FALSE);
+      g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH, FALSE);
     }
   else if (file != NULL)
     file_info_from_file (file, info);
@@ -711,13 +717,58 @@ try_create_dir_monitor (GVfsBackend *backend,
       return TRUE;
     }
   
-  g_vfs_job_create_monitor_set_obj_path (job,
-                                         g_vfs_monitor_get_object_path (computer_backend->root_monitor));
+  g_vfs_job_create_monitor_set_monitor (job,
+                                        computer_backend->root_monitor);
   g_vfs_job_succeeded (G_VFS_JOB (job));
 
   return TRUE;
 }
 
+static void
+mount_volume_cb (GObject *source_object,
+                 GAsyncResult *res,
+                 gpointer user_data)
+{
+  GVfsJobMountMountable *job = user_data;
+  GError *error;
+  GMount *mount;
+  GVolume *volume;
+  GFile *root;
+  char *uri;
+  
+  volume = G_VOLUME (source_object);
+
+  /* TODO: We're leaking the GMountOperation here */
+  
+  error = NULL;
+  if (g_volume_mount_finish (volume, res, &error))
+    {
+      mount = g_volume_get_mount (volume);
+
+      if (mount)
+        {
+          root = g_mount_get_root (mount);
+          uri = g_file_get_uri (root);
+          g_vfs_job_mount_mountable_set_target_uri (job,
+                                                    uri,
+                                                    FALSE);
+          g_free (uri);
+          g_object_unref (root);
+          g_object_unref (mount);
+          g_vfs_job_succeeded (G_VFS_JOB (job));
+        }
+      else
+        g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                          G_IO_ERROR_FAILED,
+                          _("Can't find mount for mounted volume"));
+    }
+  else
+    {
+      g_vfs_job_failed_from_error  (G_VFS_JOB (job), error);
+      g_error_free (error);
+    }
+}
+
 static gboolean
 try_mount_mountable (GVfsBackend *backend,
                      GVfsJobMountMountable *job,
@@ -725,6 +776,7 @@ try_mount_mountable (GVfsBackend *backend,
                      GMountSource *mount_source)
 {
   ComputerFile *file;
+  GMountOperation *mount_op;
 
   file = lookup (G_VFS_BACKEND_COMPUTER (backend),
                  G_VFS_JOB (job), filename);
@@ -732,25 +784,25 @@ try_mount_mountable (GVfsBackend *backend,
   if (file == &root)
     g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
                       G_IO_ERROR_NOT_MOUNTABLE_FILE,
-                      _("Can't open directory"));
+                      _("Not a mountable file"));
   else if (file != NULL)
     {
-#if 0
       if (file->volume)
         {
-          /* TODO: Implement */
+          mount_op = g_mount_source_get_operation (mount_source);
           g_volume_mount (file->volume,
-                          GMountOperation      *mount_operation,
-                          GCancellable         *cancellable,
-                          GAsyncReadyCallback   callback,
-                          gpointer              user_data);
+                          mount_op,
+                          G_VFS_JOB (job)->cancellable,
+                          mount_volume_cb,
+                          job);
         }
+#if 0
       else if (file->drive)
         {
           /* TODO: Poll for media? */
         }
-      else
 #endif
+      else
         {
           g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
                             G_IO_ERROR_NOT_SUPPORTED,
@@ -761,18 +813,178 @@ try_mount_mountable (GVfsBackend *backend,
   return TRUE;
 }
 
+static void
+unmount_mount_cb (GObject *source_object,
+		  GAsyncResult *res,
+		  gpointer user_data)
+{
+  GVfsJobMountMountable *job = user_data;
+  GError *error;
+  GMount *mount;
+  
+  mount = G_MOUNT (source_object);
+
+  error = NULL;
+  if (g_mount_unmount_finish (mount, res, &error))
+    g_vfs_job_succeeded (G_VFS_JOB (job));
+  else
+    {
+      g_vfs_job_failed_from_error  (G_VFS_JOB (job), error);
+      g_error_free (error);
+    }
+}
+
+
 static gboolean
 try_unmount_mountable (GVfsBackend *backend,
-                       GVfsJobMountMountable *job,
-                       const char *filename)
+		       GVfsJobUnmountMountable *job,
+		       const char *filename,
+		       GMountUnmountFlags flags)
 {
+  ComputerFile *file;
+
+  file = lookup (G_VFS_BACKEND_COMPUTER (backend),
+                 G_VFS_JOB (job), filename);
+  
+  if (file == &root)
+    g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                      G_IO_ERROR_NOT_MOUNTABLE_FILE,
+                      _("Not a mountable file"));
+  else if (file != NULL)
+    {
+      if (file->mount)
+        {
+          g_mount_unmount (file->mount,
+                         flags,
+                         G_VFS_JOB (job)->cancellable,
+                         unmount_mount_cb,
+                         job);
+        }
+      else
+        {
+          g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                            G_IO_ERROR_NOT_SUPPORTED,
+                            _("Can't unmount file"));
+        }
+    }
+  
+  return TRUE;
+}
+
+static void
+eject_mount_cb (GObject *source_object,
+                GAsyncResult *res,
+                gpointer user_data)
+{
+  GVfsJobMountMountable *job = user_data;
+  GError *error;
+  GMount *mount;
+  
+  mount = G_MOUNT (source_object);
+
+  error = NULL;
+  if (g_mount_eject_finish (mount, res, &error))
+    g_vfs_job_succeeded (G_VFS_JOB (job));
+  else
+    {
+      g_vfs_job_failed_from_error  (G_VFS_JOB (job), error);
+      g_error_free (error);
+    }
+}
+
+static void
+eject_volume_cb (GObject *source_object,
+                 GAsyncResult *res,
+                 gpointer user_data)
+{
+  GVfsJobMountMountable *job = user_data;
+  GError *error;
+  GVolume *volume;
+  
+  volume = G_VOLUME (source_object);
+
+  error = NULL;
+  if (g_volume_eject_finish (volume, res, &error))
+    g_vfs_job_succeeded (G_VFS_JOB (job));
+  else
+    {
+      g_vfs_job_failed_from_error  (G_VFS_JOB (job), error);
+      g_error_free (error);
+    }
+}
+
+
+static void
+eject_drive_cb (GObject *source_object,
+                GAsyncResult *res,
+                gpointer user_data)
+{
+  GVfsJobMountMountable *job = user_data;
+  GError *error;
+  GDrive *drive;
+  
+  drive = G_DRIVE (source_object);
+
+  error = NULL;
+  if (g_drive_eject_finish (drive, res, &error))
+    g_vfs_job_succeeded (G_VFS_JOB (job));
+  else
+    {
+      g_vfs_job_failed_from_error  (G_VFS_JOB (job), error);
+      g_error_free (error);
+    }
 }
 
 static gboolean
 try_eject_mountable (GVfsBackend *backend,
-                     GVfsJobMountMountable *job,
-                     const char *filename)
+                     GVfsJobUnmountMountable *job,
+                     const char *filename,
+                     GMountUnmountFlags flags)
 {
+  ComputerFile *file;
+
+  file = lookup (G_VFS_BACKEND_COMPUTER (backend),
+                 G_VFS_JOB (job), filename);
+  
+  if (file == &root)
+    g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                      G_IO_ERROR_NOT_MOUNTABLE_FILE,
+                      _("Not a mountable file"));
+  else if (file != NULL)
+    {
+      if (file->mount)
+        {
+          g_mount_eject (file->mount,
+                         flags,
+                         G_VFS_JOB (job)->cancellable,
+                         eject_mount_cb,
+                         job);
+        }
+      else if (file->volume)
+        {
+          g_volume_eject (file->volume,
+                          flags,
+                          G_VFS_JOB (job)->cancellable,
+                          eject_volume_cb,
+                          job);
+        }
+      else if (file->drive)
+        {
+          g_drive_eject (file->drive,
+                         flags,
+                         G_VFS_JOB (job)->cancellable,
+                         eject_drive_cb,
+                         job);
+        }
+      else
+        {
+          g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                            G_IO_ERROR_NOT_SUPPORTED,
+                            _("Can't eject file"));
+        }
+    }
+  
+  return TRUE;
 }
 
 static void
@@ -788,4 +1000,7 @@ g_vfs_backend_computer_class_init (GVfsBackendComputerClass *klass)
   backend_class->try_query_info = try_query_info;
   backend_class->try_enumerate = try_enumerate;
   backend_class->try_create_dir_monitor = try_create_dir_monitor;
+  backend_class->try_mount_mountable = try_mount_mountable;
+  backend_class->try_unmount_mountable = try_unmount_mountable;
+  backend_class->try_eject_mountable = try_eject_mountable;
 }
